package testen;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import domein.Account;
import domein.AccountManager;
import domein.AccountService;
import domein.IAccountManager;

import org.mockito.Mockito;

public class AccountServiceTest 
{
	private AccountService accountService;
	
	private AccountManager mockAccountManager;
	//BOVEN: indien Mockito External Jar, ONDER: indien Mock
	private MockAccountManager mockDummy;
	
	
	private final static String SENDER_ID = "1", BENIF_ID = "2";
	
	@Before
	public void before()
	{
		/*
		//(Zonder Mockito)
		mockDummy = new MockAccountManager();
		accountService = new AccountService();
		accountService.setAccountManager(mockDummy);
		*/

		//Met Mockito:
		mockAccountManager = Mockito.mock(AccountManager.class);//Mockito object aanmaken voor de DB simulatie
		accountService = new AccountService();//Landservice object aanmaken
		accountService.setAccountManager(mockAccountManager); //setter injection, de dummy injecteren in het programma
		//hollywood principe (inversion of control (IOC))

	}
	
	@Test
	public void accountServiceTestGoed()
	{
		/*
		//!ZONDER MOCKITO JAR: (External Jar)
		//mock trainen:
		//2 account objecten maken <String, Account>
		mockDummy.addAccount(SENDER_ID, new Account(SENDER_ID, 200));
		mockDummy.addAccount(BENIF_ID, new Account(BENIF_ID, 100));
		
		//verwerken:
		accountService.transfer(SENDER_ID, BENIF_ID, 50);

		//uitkomst controleren met Assert:
		Assert.assertEquals(150, mockDummy.findAccountForUser(SENDER_ID).getBalance());
		Assert.assertEquals(150, mockDummy.findAccountForUser(BENIF_ID).getBalance());
		*/

		//!MET MOCKITO JAR: (External Jar)
		//De twee objecten:
		Account accounta = new Account(SENDER_ID, 200);
		Account accountb= new Account(BENIF_ID, 100);
		
		//1 Mock trainen:
		Mockito.when(mockAccountManager.findAccountForUser(SENDER_ID)).thenReturn(accounta);
		Mockito.when(mockAccountManager.findAccountForUser(BENIF_ID)).thenReturn(accountb);
		
		//2 Mock verwerking:
		accountService.transfer(SENDER_ID, BENIF_ID, 50);
		
		//3 Verify: (Verify voor Controle is altijd goed!)
		Mockito.verify(mockAccountManager).findAccountForUser(SENDER_ID);
		Mockito.verify(mockAccountManager).findAccountForUser(BENIF_ID);
		Mockito.verify(mockAccountManager).updateAccount(accounta);
		Mockito.verify(mockAccountManager).updateAccount(accountb);
		
		//4 Controle:
		Assert.assertEquals(150, mockAccountManager.findAccountForUser(SENDER_ID).getBalance());
		Assert.assertEquals(150, mockAccountManager.findAccountForUser(BENIF_ID).getBalance());

	}

}
